using System;
using System.Collections.Generic;
using System.Drawing;

using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.DataVisualization;

public class VLModule : IVLModule {
    public void Run() {
      //------------------------------------
      //- Parameters                       -
      //------------------------------------
        // refractive index of upper layer
        double n1 = 1.5;
        // refractive index of lower layer
        double n2 = 2.2;
        // DOE definition area (DOE aperture diameter)
        double DOEDefinitionArea = 1e-3;
      
      
      //------------------------------------
      //- Calculation of Surface Profiles  -
      //------------------------------------
        ComplexAmplitude transmission = (ComplexAmplitude )Globals.ActiveDocumentHistory.GetLastDocument(0, DocumentFilter.CATransmission);
        ComplexAmplitude height1 = new ComplexAmplitude(transmission);
        Vector samplingPoints = (Vector )(new VectorD(DOEDefinitionArea, DOEDefinitionArea) / height1.SamplingDistance);
        if (MathFunctions.IsEven(samplingPoints.X)){
          samplingPoints.X += 1;
        }
        if (MathFunctions.IsEven(samplingPoints.Y)){
          samplingPoints.Y += 1;
        }
        height1.Field.Replicate_Periodic(samplingPoints);
        height1.ModifyComplexPart(ComplexPart.ShiftPhaseToReal);
        height1.MultEqual(transmission.Wavelength / 2.0 / MathFunctions.TwoPi / (n1 - n2));
        height1.LiftPositive();
        DiagramData heightDiagr1 = new DiagramData(height1.Field); 
        VectorD min = new VectorD(-1.0,-1.0)*(height1.SamplingPoints / new Vector(2, 2)) * height1.SamplingDistance;
        VectorD max = min + height1.SamplingPoints * height1.SamplingDistance;
        heightDiagr1.MinX = min.X;
        heightDiagr1.MaxX = max.X;
        heightDiagr1.MinY = min.Y;
        heightDiagr1.MaxY = max.Y;
        heightDiagr1.Interpolation = DiagramInterpolation.Pixelated;
        heightDiagr1.ColorMapPalette = ColorMapPalette.BlackWhite;
        Globals.DataDisplay.ShowDocument(heightDiagr1, "Height Profile Forward Propagation");
        
      
        ComplexAmplitude height2 = new ComplexAmplitude(height1);
        height2.MultEqual(-1);
        height2.Field.HorizontalMirror();
        height2.LiftPositive();
        DiagramData heightDiagr2 = new DiagramData(height2.Field); 
        heightDiagr2.MinX = min.X;
        heightDiagr2.MaxX = max.X;
        heightDiagr2.MinY = min.Y;
        heightDiagr2.MaxY = max.Y;
        heightDiagr2.Interpolation = DiagramInterpolation.Pixelated;
        heightDiagr2.ColorMapPalette = ColorMapPalette.BlackWhite;
        Globals.DataDisplay.ShowDocument(heightDiagr2, "Height Profile Backward Propagation");
    }
}
